#include "Cmd.h"

void IGL::NormalVec ( const float PRef[3], const float P1[3],
		      const float P2[3], float Nrml[3] )
{
  float v1[3], v2[3], c;

  // Differenzen vi=Pi-PRef
  v1[0] = P1[0]-PRef[0];  v2[0] = P2[0]-PRef[0];
  v1[1] = P1[1]-PRef[1];  v2[1] = P2[1]-PRef[1];
  v1[2] = P1[2]-PRef[2];  v2[2] = P2[2]-PRef[2];

  // Kreuzprodukt Nrml=v1*v2 (ist orthogonal zu v1 und v2)
  Nrml[0] = v1[1]*v2[2] - v2[1]*v1[2];
  Nrml[1] = v1[2]*v2[0] - v2[2]*v1[0];
  Nrml[2] = v1[0]*v2[1] - v2[0]*v1[1];

  // Normiere Nrml
  if ((c=sqrt(Nrml[0]*Nrml[0]+Nrml[1]*Nrml[1]+Nrml[2]*Nrml[2])))
    {
      // Dividiere durch Laenge der Normale, falls sie nicht verschwindet.
      // Kehre vorher aber Richtung um, falls Normale vom Betrachter weg zeigt.
      if (Nrml[2]<0) c=-c;
      Nrml[0]/=c; Nrml[1]/=c; Nrml[2]/=c;
    }
  else
    {
#ifdef IGL_Debug
      IGLWarn("Degenerate normal vector!");
#endif
      Nrml[0]=1.0;                        // sonst setze Nrml=(1,0,0).
    }
}

void CmdWin::Exec ( BaseWin &Win )
{
  switch (type)
    {
    case Opn:
      Win.open();
      break;

    case Cls:
      Win.close();
      break;

    case Clr:
      IGL_DEBUG("Clr issued" << endl);
      Win.clear();
      // Fenster ggf. neu zeichnen
      if (Win.isOpen && Win.drawingMode!=Buffer) Win.redisplay();
      break;
      
    case Res:
      Win._reshape (Win.width, Win.height);
      break;

    case Rdp:
      IGL_DEBUG("redisplay issued" << endl);
      break;
    }

  // zusaetzlich beim GLUT einen redisplay bewirken
  glutPostRedisplay();
}


CmdSetProp::CmdSetProp ( int Ty, float Pr0, float Pr1, float Pr2, float Pr3,
                         float Pr4, float Pr5, int IPr, const char Str[] )
  : type(Ty)
{
  switch (type)
    {
    case SPntS:
    case SMarkS:
    case SLineW:
      prop[0]=Pr0;
      break;

    case SAxis:
      prop[0]=Pr0;
      prop[1]=Pr1;
      prop[2]=Pr2;
      prop[3]=Pr3;
      prop[4]=Pr4;
      prop[5]=Pr5;
      break;

    case SDMod:
    case SWMod:
      iprp=IPr;
      break;

    case Col:
      prop[3]=1.0;
    case BCol:
      prop[0]=Pr0;
      prop[1]=Pr1;
      prop[2]=Pr2;
      break;

    case WNam:
      string=Str?strcpy(new char[strlen(Str)+1],Str):NULL;
      break;
    }
}


void CmdSetProp::Exec ( BaseWin &Win )
{
  switch (type)
    {
    case SPntS:
      glPointSize (prop[0]);
      break;

    case SMarkS:
      Win.markerSize=prop[0];
      break;

    case SLineW:
      glLineWidth (prop[0]);
      break;

    case SAxis:
      for ( int i=0; i<6; i++ )
        Win.axis[i]=prop[i];
      if (Win.isOpen && Win.drawingMode!=Buffer) Win.redisplay();
      break;

    case SDMod:
      if (Win.drawingMode!=DrawMode(iprp)
          && (Win.drawingMode=DrawMode(iprp))!=Buffer) Win.redisplay();
      break;

    case SWMod:
      if (Win.wireMode!=bool(iprp))
        { Win.wireMode=bool(iprp); Win.redisplay(); }
      break;

    case Col:
      glColor4fv(prop);
      glMaterialfv(GL_FRONT, GL_DIFFUSE, prop);
      glMaterialfv(GL_FRONT, GL_SPECULAR, prop);
      break;

    case BCol:
      Win.backColor[0]=prop[0];
      Win.backColor[1]=prop[1];
      Win.backColor[2]=prop[2];
      if (Win.isOpen && Win.drawingMode!=Buffer) Win.redisplay();
      break;

    case WNam:
      if (Win.name!="IGL")
        delete[] Win.name;
      Win.name=string?string:"IGL";
      break;

#ifdef IGL_Debug
    default:
      IGLWarn("Illegal property number!");
#endif
    }
}


void CmdObj::Exec ( BaseWin &Win )
{
  if (mark<0){
    switch (type)
      {
      case Pnt:
        glBegin     (GL_POINTS);
        glNormal3f  (0,0,1);
        glVertex3fv (coord);
        glEnd       ();
        break;

      case Line:
        glBegin     (GL_LINES);
        glNormal3f  (0,0,1);
        glVertex3fv (coord);
        glVertex3fv (coord+3);
        glEnd       ();
        break;

      case TriFlat:
        glBegin     (GL_POLYGON);
        glNormal3f  (0,0,1);
        glVertex3fv (coord);
        glVertex3fv (coord+3);
        glVertex3fv (coord+6);
        glEnd       ();
        break;

      case Tri:
        if (!nrml)
	  {
	    nrml=new float[9];
	    NormalVec(coord,coord+3,coord+6,nrml);
	    nrml[3]=nrml[0]; nrml[4]=nrml[1]; nrml[5]=nrml[2];
	    nrml[6]=nrml[0]; nrml[7]=nrml[1]; nrml[8]=nrml[2];
	  }
        glBegin     (GL_POLYGON);
        glNormal3fv (nrml);
        glVertex3fv (coord);
        glNormal3fv (nrml+3);
        glVertex3fv (coord+3);
        glNormal3fv (nrml+6);
        glVertex3fv (coord+6);
        glEnd       ();
        break;

      case Rect:
        glNormal3f(0,0,1);
        glRectfv (coord, coord+2);
        break;

      case Quad:
        if (!nrml)
	  {
	    nrml=new float[12];
	    NormalVec(coord,coord+9,coord+3,nrml);
	    NormalVec(coord+3,coord,coord+6,nrml+3);
	    NormalVec(coord+6,coord+3,coord+9,nrml+6);
	    NormalVec(coord+9,coord+6,coord,nrml+9);
	  }
        glBegin     (GL_POLYGON);
        glNormal3fv (nrml);
        glVertex3fv (coord);
        glNormal3fv (nrml+3);
        glVertex3fv (coord+3);
        glNormal3fv (nrml+6);
        glVertex3fv (coord+6);
        glNormal3fv (nrml+9);
        glVertex3fv (coord+9);
        glEnd       ();
        break;

#ifdef IGL_Debug
      default:
        IGLWarn("Illegal object number!");
#endif
      }}
  else
    switch (mark)
      {
      case Crss:
        glBegin     (GL_LINES);
        glNormal3f  (0,0,1);
        glVertex3f  (coord[0]-Win.markerSize/2*(Win.axis[1]-Win.axis[0]),coord[1]-Win.markerSize/2*(Win.axis[3]-Win.axis[2]),0);
        glVertex3f  (coord[0]+Win.markerSize/2*(Win.axis[1]-Win.axis[0]),coord[1]+Win.markerSize/2*(Win.axis[3]-Win.axis[2]),0);
        glVertex3f  (coord[0]-Win.markerSize/2*(Win.axis[1]-Win.axis[0]),coord[1]+Win.markerSize/2*(Win.axis[3]-Win.axis[2]),0);
        glVertex3f  (coord[0]+Win.markerSize/2*(Win.axis[1]-Win.axis[0]),coord[1]-Win.markerSize/2*(Win.axis[3]-Win.axis[2]),0);
        glEnd       ();
        break;

      case Plus:
        glBegin     (GL_LINES);
        glNormal3f  (0,0,1);
        glVertex3f  (coord[0]-Win.markerSize/2*(Win.axis[1]-Win.axis[0]),coord[1],0);
        glVertex3f  (coord[0]+Win.markerSize/2*(Win.axis[1]-Win.axis[0]),coord[1],0);
        glVertex3f  (coord[0],coord[1]+Win.markerSize/2*(Win.axis[3]-Win.axis[2]),0);
        glVertex3f  (coord[0],coord[1]-Win.markerSize/2*(Win.axis[3]-Win.axis[2]),0);
        glEnd       ();
        break;

      case Sqr:
        glBegin     (GL_LINES);
        glNormal3f  (0,0,1);
        glVertex3f  (coord[0]-Win.markerSize/2*(Win.axis[1]-Win.axis[0]),coord[1]-Win.markerSize/2*(Win.axis[3]-Win.axis[2]),0);
        glVertex3f  (coord[0]-Win.markerSize/2*(Win.axis[1]-Win.axis[0]),coord[1]+Win.markerSize/2*(Win.axis[3]-Win.axis[2]),0);
        glVertex3f  (coord[0]+Win.markerSize/2*(Win.axis[1]-Win.axis[0]),coord[1]+Win.markerSize/2*(Win.axis[3]-Win.axis[2]),0);
        glVertex3f  (coord[0]+Win.markerSize/2*(Win.axis[1]-Win.axis[0]),coord[1]-Win.markerSize/2*(Win.axis[3]-Win.axis[2]),0);
        glEnd       ();
        break;

      case Rhmb:
        glBegin     (GL_LINE_LOOP);
        glNormal3f  (0,0,1);
        glVertex3f  (coord[0]-Win.markerSize/2*(Win.axis[1]-Win.axis[0]),coord[1],0);
        glVertex3f  (coord[0],coord[1]-Win.markerSize/2*(Win.axis[3]-Win.axis[2]),0);
        glVertex3f  (coord[0]+Win.markerSize/2*(Win.axis[1]-Win.axis[0]),coord[1],0);
        glVertex3f  (coord[0],coord[1]+Win.markerSize/2*(Win.axis[3]-Win.axis[2]),0);
        glEnd       ();
        break;

      default:
        IGLWarn("Marker noch nicht implementiert!");
      }
}
