#include "BaseWin.h"
#include "IGL_intern.h"

#include <queue>

using namespace IGL;

void IGL::NormalVec ( const float PRef[3], const float P1[3],
		      const float P2[3], float Nrml[3] );
// ****************************************************************************
// *                Befehls-Klasse und abgeleitet Klassen                     *
// ****************************************************************************

// Die Befehls-Klasse ist eine (abstrakte) Klasse zum Speichern von
// Grafik-Befehlen. Von ihr werden fuer verschiedene Befehle bzw.
// Befehlsgruppen angepasste Klassen abgeleitet. Den einzelnen Funktionen
// kommt dabei folgende Bedeutung zu:
//
// Der Konstruktor speichert die erforderlichen Daten, "Exec" (wird von 
// display() aus aufgerufen) fuehrt den Befehl aus, und der Destruktor
// gibt gegebenenfalls allozierten Speicher wieder frei.

class IGL::IGLCmd
{
  protected:
    virtual ~IGLCmd () {}
    virtual void Exec ( BaseWin& ) = 0;

  friend class BaseWin;
  friend class CmdWin;
  friend class IGLoop;
};


// ----- Fenster manipulieren -----

class IGL::CmdWin : public IGL::IGLCmd
{
  private:
    int  type;
    uint width, height;
  protected:
    CmdWin  ( int Ty, uint W=0, uint H=0 ) : type(Ty), width(W), height(H) {}
    ~CmdWin () {}
    void Exec ( BaseWin& );

  friend class BaseWin;
  friend class IGLWin;
};


// ----- Eigenschaft (Punktgroesse, Farbe etc.) setzen -----

class IGL::CmdSetProp : public IGL::IGLCmd
{
  private:
    int   type, iprp;
    float prop[6];
    char  *string;
  protected:
    CmdSetProp ( int Ty, float Pr0, float Pr1=0, float Pr2=0, float Pr3=0,
                 float Pr4=0, float Pr5=0, int IPr=0, const char Str[]=NULL );
    ~CmdSetProp () {}
    void Exec ( BaseWin& );

  friend class IGLWin;
};


// ----- Grafikobjekt zeichnen -----

class IGL::CmdObj : public IGL::IGLCmd
{
  private:
    int   type, mark;
    float *coord, *nrml;
  protected:
    CmdObj ( int Ty, int Mk, float *Co, float *No=NULL )
      : type(Ty), mark(Mk), coord(Co), nrml(No) {}
    ~CmdObj () { delete[] coord; if (nrml) delete[] nrml; }
    void Exec ( BaseWin& );

  friend class IGLWin;
};


